//ODEHuman.h
//--------------------** Description ** -----------------------------------------------
// A set of joints and rigid boxes representing a human figure
//-------------------------------------------------------------------------------------
//Copyright 2005 Michael J Mandel
//Licensed under the LGPL (see license.txt for more information)

#ifndef _ODEHUMAN_H__
#define _ODEHUMAN_H__

#include <vector>
#include "ODEObject.h"
#include "TimeStamp.h"
#include "ODEHumanProperties.h"

class ODEBox;
class ODESimulation;


class ODEHuman : public ODEObject
{
	public:
	ODEHuman(const ODEPosition &humanPos, ODESimulation *sim, bool hasBody = true);
	~ODEHuman();

	virtual void Render();
	virtual bool Update();

	//enable/disable all bodies and joints
	void SetEnabled(bool b);
	bool isEnabled() { return mEnabled; }

	//add a force to a particular joint
	void AddForceToBody(int bodyID, const ODEVector &force);

	//access an individual body part
	ODEBox * GetBody(int i) { 
			return rigidLinks[i];
	}

	//enable PD-controller
	void EnableController(bool b);
	bool isControllerEnabled() { return mControllerEnabled; }

	private:
	bool mEnabled;

	//set the controller gains used by the controller
	void SetControllerProperties(odeControlProperty *props);

	//creates all the joints and bodies for a human in the reference pose
	void CreateBodiesAndJoints(const ODEPosition &humanPos, bool allowAttachToWorld = true);

	//rigid bodies and joints making up the structure of the human body
	std::vector<ODEBox*> rigidLinks;
	std::vector<dJointID> rigidJoints;

	bool mControllerEnabled;
	//compute and apply controller torques
	void applyControlTorques();

	//current controller gain specification
	odeControlProperty *mCurControllerProps;
};

#endif